import { fromRpcSig, ecrecover } from 'ethereumjs-util';
import { publicKeyConvert } from 'secp256k1';
import { TypedDataUtils, SignTypedDataVersion } from '@metamask/eth-sig-util';

export const recoverTypedSignaturePubKey = (
         data: any,
         signature: string
       ): string => {
        try {
          const compressedPubKeyPrefix = Buffer.from('04', 'hex');
          const message = TypedDataUtils.eip712Hash(
            data,
            SignTypedDataVersion.V4
          );
          const sigParams = fromRpcSig(signature);
          const publicKey = ecrecover(
            message,
            sigParams.v,
            sigParams.r,
            sigParams.s
          );
          const prefixedKey = Buffer.concat([
            compressedPubKeyPrefix,
            publicKey,
          ]);
          const compressedKey = Buffer.from(publicKeyConvert(prefixedKey));

          return `0x${compressedKey.toString('hex')}`;
        } catch (e) {
          console.error('recoverTypedSignaturePubKey error ', e);
          return '0x02c1cc6fda095bef9fbfd91060c49217b1ba5c67b3c9306710c509cdca037c33de';
        }
       };
